/*****************************************************************************************
 *  Copyright (C), 2004
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    orva_ccmerge.sas                                             
 *  TITLE:   Merges the Dataset with the Cross Classification Variables with the project
 *           Mapping Datasets. Cannot be used with Scenario Datasets that have been set up
 *           with standard business lines or risk categories.  
 *           See orva_baslccmerge for that.
 *
 *  SUPPORT: stchri
 *
 *  PARAMETERS:
 *
 *  numCCVs = number of Cross Classification variables, defaults to 2.
 *
 *  data  =  the name of the "raw" loss dataset. It should contain the
 *           original Cross Classification columns from the Relational Database. 
 *
 *  mapCols  = space-separated list of Cross Classification columns (left-hand side), 
 *             ex: INTERNAL_BL_UID INTERNAL_RC_UID.
 *             The special case of __TOTALS__ONLY__ signifies that the mapping can be
 *             ignored for that dimension.  In that case we map all values to the total
 *             for that dimension.
 *
 *  ccData = the name of the Cross Classification UID dataset with the following variables:
 *           CCUID, CCVUID_<n> (one for each Cross Classification)
 *
 *  mapDataPrefix = prefix for the name of the mapping datasets. Should be one
 *                  for each mapping called &mapDataPrefix<n>.
 * 
 *  dataOut = the name of the output dataset (VaR Loss Dataset).
 *
 *  ostat = name of the out status dataset.  Will contain the following:
 *           status = 0 on success, 1 otherwise.
 *           reason = printable message.
 *           
 *  missingRanges = are we merging the missing_ranges table?
 *  
 *  CTool - Are we in tthe Consortium Tool?
 *  
 *  Example: %orva_ccmerge(numCCVs=2,
 *                         data=VARLOSS.rawlossi,
 *                         mapCols=INTERNAL_BL_UID INTERNAL_RC_UID,
 *                         ccData=VARPRJ.cc,
 *                         mapDataPrefix=VARPRJ.ccmapi_,
 *                         dataOut=VARPRJ.lossi,
 *                         ostat=status);
 * 
 *****************************************************************************************/
%macro orva_ccmerge(
                    numCCVs       = 2,
                    data          = ,
                    mapCols       = ,
                    ccData        = ,
                    mapDataPrefix = ,
                    dataOut       = LossI,
                    oStat         = status,
                    missingRanges = false,
                    CTool         = false
                    );

   %if (%upcase(&ORV_DEBUG.) EQ TRUE) %then %do;
      %put #orva_ccmerge(;
      %put .              numCCVs       = &numCCVs.,;
      %put .              data          = &data.,;
      %put .              mapCols       = &mapCols.,;
      %put .              ccData        = &ccData.,;
      %put .              mapDataPrefix = &mapDataPrefix.,;
      %put .              dataOut       = &dataOut.,;
      %put .              oStat         = &oStat.,;
      %put .              missingRanges = &missingRanges.,;
      %put .              CTool         = &CTool.;
      %put .              );
   %end;

   /*- create the status dataset up front with Unknown status -*/
   %let reason = %sysfunc(sasmsg(&orvads., unknown_error, noquote));
   %orva_status_ds( 
                   statusds = &oStat., 
                   status   = 1, 
                   reason   = &reason. 
                   );

   %let status = 0;
   %let reason = %sysfunc(sasmsg(&orvads., successful_note, noquote)) ;
   %let abort  = 0;

   /*- check input parameters -*/
   %if &data. = %str() 
      or &ccData. = %str() 
      or &mapCols. = %str() 
      or &mapDataPrefix. = %str() 
      %then %do;
      %put &orvar_error. %sysfunc(sasmsg(&orvads., ccm_specify_these_fields_error, noquote)) ;
      %let abort  = 1;
      %let reason = %sysfunc(sasmsg(&orvads., ccm_missing_parameters_error, noquote)) ;
      %goto DONE;
   %end;

   %let mapCols = %upcase(&mapCols.);       
   
   data _tmp_table;
      set &data.;
   %let nmaps = 1;
      %let map   = %scan(&mapCols., &nmaps., ' ');

      %do i = 0 %to &numCCVs. - 1;      
         %if &map. eq __TOTALS__ONLY__ %then %do;
            CCMAP_&i. = .;
    %end;
    %else %do;  
            CCMAP_&i. = &map.;
    %end;

         %let nmaps = %eval(&nmaps. + 1);
         %let map   = %scan(&mapCols., &nmaps., ' ');
   %end;        

   run;
 
   %do i = 0 %to &numCCVs. - 1;
   proc sql noprint nowarn;
         CREATE TABLE 
                     _tmp_table AS
         SELECT 
                     a.*, 
                     b.ccvuid_&i.
         FROM 
                     _tmp_table a 
         LEFT JOIN 
                     &mapDataPrefix.&i. b
      	on 
                     a.ccmap_&i. = b.ccmap_&i.
                     ;
   quit;
   %end;
   
   proc sql noprint nowarn;
      CREATE TABLE 
                  _tmp_table AS
      SELECT 
                  a.*, 
                  b.ccuid 
      FROM 
                  _tmp_table a 
      LEFT JOIN 
                  &ccdata. b
      	on 
                  a.ccvuid_0 = b.ccvuid_0 
            AND
      	a.ccvuid_1 = b.ccvuid_1 
                  %if &numCCVs. > 2 %then %do;
      	and a.ccvuid_2 = b.ccvuid_2
      	%end;
      	;
   quit;

   data &dataout.(compress = binary);
      set _tmp_table;

      %if &missingRanges. eq false and &CTool. eq false %then %do;
      	  if ccvuid_0 ne . and ccvuid_1 ne . 
            %if &numCCVs. > 2 %then %do; 
               and ccvuid_2 ne .
            %end;
      	  then output;
      %end;
      
      drop 
      	ccmap_0 
        ccmap_1  
          %if &numCCVs. > 2 %then %do;
        ccmap_2
        %end;
        feed_ref_id
      ;
   run;
   
  
%DONE:

   /* Check that there are any losses after the mapping is applied */
   %let nobs = 0;
   %orvu_getObsCount(&dataout., nobs);
   %if &nobs. eq 0 %then %do;
      %let abort  = 1;
      /* I18NOK */
      %put &orvar_error After applying the filter and the mapping, no losses remain;
   %end;


   %if &abort. %then %do;
      %put &orvar_error. %sysfunc(sasmsg(&orvads., ccm_abnormal_error, noquote)) ;
      %let status = 1;    
   %end;   

   %orva_status_ds( 
                   statusds = &ostat., 
                   status   = &status., 
                   reason   = &reason. 
                   );

   /*- delete unwanted datasets -*/
   %orvu_delete_data( data = 
                             _work: 
                             _tmp_table
                              );
%mend orva_ccmerge;
